from marshmallow import post_load, fields
from sqlalchemy import orm
import uuid

from core.managers.log_manager import logger
from core.managers.db_manager import db
from core.model.parameter_value import ParameterValueImportSchema

from core.model.publishers_node import PublishersNode
from shared.schema.publisher_preset import (
    PublisherPresetSchema,
    PublisherPresetPresentationSchema,
)


class NewPublisherPresetSchema(PublisherPresetSchema):
    parameter_values = fields.List(fields.Nested(ParameterValueImportSchema))

    @post_load
    def make(self, data, **kwargs):
        return PublisherPreset(**data)


class PublisherPreset(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    publisher_id = db.Column(db.String, db.ForeignKey("publisher.id"))
    publisher = db.relationship("Publisher", back_populates="presets")

    use_for_notifications = db.Column(db.Boolean)

    parameter_values = db.relationship("ParameterValue", secondary="publisher_preset_parameter_value", cascade="all")

    def __init__(
        self,
        id,
        name,
        description,
        publisher_id,
        parameter_values,
        use_for_notifications=False,
    ):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.publisher_id = publisher_id
        self.parameter_values = parameter_values
        self.use_for_notifications = use_for_notifications
        self.tag = "mdi-file-star-outline"

    @orm.reconstructor
    def reconstruct(self):
        self.tag = "mdi-file-star-outline"

    @classmethod
    def find(cls, preset_id):
        return cls.query.get(preset_id)

    @classmethod
    def find_for_notifications(cls):
        return cls.query.filter_by(use_for_notifications=True).first()

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(PublisherPreset.name)).all()

    @classmethod
    def get(cls, search=None):
        query = cls.query

        if search:
            query = query.filter(
                db.or_(
                    cls.name.ilike(f"%{search}%"),
                    cls.description.ilike(f"%{search}%"),
                )
            )

        return query.order_by(cls.name).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        publishers, count = cls.get(search)
        publisher_schema = PublisherPresetPresentationSchema(many=True)
        return {"total_count": count, "items": publisher_schema.dump(publishers)}

    @classmethod
    def get_all_for_publisher_json(cls, parameters):
        node = PublishersNode.get_by_api_key(parameters.api_key)
        for publisher in node.publishers:
            if publisher.type == parameters.publisher_type:
                presets_schema = PublisherPresetSchema(many=True)
                return presets_schema.dump(publisher.sources)

    @classmethod
    def add_new(cls, data):
        logger.debug(f"Adding new publisher preset: {data}")
        new_preset_schema = NewPublisherPresetSchema()
        preset = new_preset_schema.load(data)
        db.session.add(preset)
        db.session.commit()

    @classmethod
    def delete(cls, preset_id):
        preset = cls.query.get(preset_id)
        db.session.delete(preset)
        db.session.commit()

    @classmethod
    def update(cls, preset_id, data):
        new_preset_schema = NewPublisherPresetSchema()
        updated_preset = new_preset_schema.load(data)
        preset = cls.query.get(preset_id)
        preset.name = updated_preset.name
        preset.description = updated_preset.description

        for value in preset.parameter_values:
            for updated_value in updated_preset.parameter_values:
                if value.parameter_key == updated_value.parameter_key:
                    value.value = updated_value.value

        db.session.commit()


class PublisherPresetParameterValue(db.Model):
    publisher_preset_id = db.Column(db.String, db.ForeignKey("publisher_preset.id"), primary_key=True)
    parameter_value_id = db.Column(db.Integer, db.ForeignKey("parameter_value.id"), primary_key=True)



from marshmallow import post_load
import uuid
from sqlalchemy import or_, func

from core.managers.db_manager import db
from core.model.parameter import Parameter
from shared.schema.publisher import PublisherSchema


class NewPublisherSchema(PublisherSchema):
    @post_load
    def make(self, data, **kwargs):
        return Publisher(**data)


class Publisher(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    type = db.Column(db.String(), nullable=False)

    parameters = db.relationship("Parameter", secondary="publisher_parameter")

    node_id = db.Column(db.String, db.ForeignKey("publishers_node.id"))
    node = db.relationship("PublishersNode", back_populates="publishers")

    presets = db.relationship("PublisherPreset", back_populates="publisher")

    def __init__(self, name, description, type, parameters):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.type = type
        self.parameters = parameters

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(Publisher.name).like(search_string),
                    func.lower(Publisher.description).like(search_string),
                )
            )

        return query.order_by(db.asc(Publisher.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        publisher, count = cls.get(search)
        node_schema = PublisherSchema(many=True)
        items = node_schema.dump(publisher)

        return {"total_count": count, "items": items}

    @classmethod
    def create_all(cls, publishers_data):
        new_publisher_schema = NewPublisherSchema(many=True)
        return new_publisher_schema.load(publishers_data)

    @classmethod
    def add(cls, data):
        if cls.find_by_type(data["type"]):
            return None
        schema = NewPublisherSchema()

        parameters = [Parameter.find_by_key(p) for p in data["parameters"] if p is not None]
        data["parameters"] = []
        publisher = schema.load(data)
        publisher.parameters = parameters

        db.session.add(publisher)
        db.session.commit()

    @classmethod
    def get_first(cls):
        return cls.query.first()

    @classmethod
    def find_by_id(cls, id):
        return cls.query.filter_by(id=id).first()

    @classmethod
    def find_by_type(cls, type):
        return cls.query.filter_by(type=type).first()


class PublisherParameter(db.Model):
    publisher_id = db.Column(db.String, db.ForeignKey("publisher.id"), primary_key=True)
    parameter_key = db.Column(db.String, db.ForeignKey("parameter.key"), primary_key=True)



from marshmallow import fields, post_load
from sqlalchemy import or_, func
import uuid

from core.managers.db_manager import db
from core.model.parameter_value import ParameterValueImportSchema
from core.managers.log_manager import logger
from shared.schema.bot import BotSchema


class NewBotSchema(BotSchema):
    parameter_values = fields.List(fields.Nested(ParameterValueImportSchema), load_default=[])

    @post_load
    def make(self, data, **kwargs):
        return Bot(**data)


class Bot(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    type = db.Column(db.String(64), nullable=False)
    parameter_values = db.relationship("ParameterValue", secondary="bot_parameter_value", cascade="all")

    def __init__(self, name, description, type, parameter_values):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.type = type
        self.parameter_values = parameter_values

    @classmethod
    def create_all(cls, bots_data):
        new_bot_schema = NewBotSchema(many=True)
        return new_bot_schema.load(bots_data)

    @classmethod
    def update_bot_parameters(cls, bot_id, data):
        try:
            bot = cls.find_by_id(bot_id)
            # parameter_values = [NewParameterValueSchema().load(pv) for pv in data["parameter_values"]]
            for pv in bot.parameter_values:
                for updated_value in data["parameter_values"]:
                    if pv.parameter.key == updated_value["parameter"]:
                        pv.value = updated_value["value"]

            # bot_params = [{"id": pv.id, "key": pv.parameter.key, "value": pv.value} for pv in bot.parameter_values]
            # print(f"bot_params = {bot_params}")
            # bot.parameter_values = parameter_values
            db.session.commit()
        except Exception:
            logger.log_debug_trace("Update Bot Parameters Failed")

    @classmethod
    def add(cls, data):
        if cls.find_by_type(data["type"]):
            return None
        schema = NewBotSchema()
        bot = schema.load(data)
        db.session.add(bot)
        db.session.commit()

    @classmethod
    def get_first(cls):
        return cls.query.first()

    @classmethod
    def find_by_id(cls, id):
        return cls.query.filter_by(id=id).first()

    @classmethod
    def find_by_type(cls, type):
        return cls.query.filter_by(type=type).first()

    @classmethod
    def get_all_by_type(cls, type):
        return cls.query.filter_by(type=type).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(Bot.name).like(search_string),
                    func.lower(Bot.description).like(search_string),
                )
            )

        return query.order_by(db.asc(Bot.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        bots, count = cls.get(search)
        node_schema = BotSchema(many=True)
        items = node_schema.dump(bots)

        return {"total_count": count, "items": items}


class BotParameterValue(db.Model):
    bot_id = db.Column(db.String, db.ForeignKey("bot.id"), primary_key=True)
    parameter_value_id = db.Column(db.Integer, db.ForeignKey("parameter_value.id"), primary_key=True)



from marshmallow import fields, post_load
from sqlalchemy import or_, func
import uuid

from core.managers.db_manager import db
from shared.schema.collector import CollectorSchema
from core.model.parameter import Parameter


class NewCollectorSchema(CollectorSchema):
    @post_load
    def make_collector(self, data, **kwargs):
        return Collector(**data)


class Collector(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    type = db.Column(db.String(), nullable=False)
    parameters = db.relationship("Parameter", secondary="collector_parameter", cascade="all")
    sources = db.relationship("OSINTSource", backref="collector")

    def __init__(self, name, description, type, parameters):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.type = type
        self.parameters = parameters

    @classmethod
    def create_all(cls, collectors_data):
        new_collector_schema = NewCollectorSchema(many=True)
        return new_collector_schema.load(collectors_data)

    @classmethod
    def add(cls, data):
        if cls.find_by_type(data["type"]):
            return None
        schema = NewCollectorSchema()
        parameters = [Parameter.find_by_key(p) for p in data["parameters"] if p is not None]
        data["parameters"] = []
        collector = schema.load(data)
        collector.parameters = parameters
        db.session.add(collector)
        db.session.commit()

    @classmethod
    def get_first(cls):
        return cls.query.first()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(Collector.name).like(search_string),
                    func.lower(Collector.description).like(search_string),
                )
            )

        return query.order_by(db.asc(Collector.name)).all(), query.count()

    @classmethod
    def find_by_type(cls, type):
        return cls.query.filter_by(type=type).first()

    @classmethod
    def get_all_json(cls, search):
        collectors, count = cls.get(search)
        node_schema = CollectorSchema(many=True)
        items = node_schema.dump(collectors)

        return {"total_count": count, "items": items}


class CollectorParameter(db.Model):
    collector_id = db.Column(db.String, db.ForeignKey("collector.id", ondelete="CASCADE"), primary_key=True)
    parameter_id = db.Column(db.String, db.ForeignKey("parameter.key", ondelete="CASCADE"), primary_key=True)
